NB. Plot examples

f=. jpath '~system\packages\graphics\'&, @ (,&'.ijs')

DATHILO=: }. 'm' fread jpath '~system\examples\data\dm0396.txt'

NB. ====================================================
NB. cubicspline        - calculate cubic spline
NB. y. is a 2-row matrix x ,: f(x)
NB. result is x values;coefficient matrix,one row per interval.
cubicspline=: 3 : 0
diff=. }.-}:
'x y'=. y.
h=. diff x
k=. diff y
w=. 3 * diff k % *: h
n=. +:(2}.x)-_2}.x
sm=. ,~_2+#x
m=. sm$}.,(sm+0 1){.(}:h),.n,.}.h
c=. 0,w %. m
a=. }:y
b=. (k%h) - h * ((+:c) + }.c,0) % 3
d=. (diff c,0) % 3 * h
(}:x);a,.b,.c,.d
)

NB. ====================================================
NB. interspline        - interpolate spline
NB. x. is a result from cubicspline
NB. y. is a set of x coordinates
NB. returns corresponding f(x) values
interspline=: 4 : 0
'i m'=. x.
n=. <: +/ i <:/ y.
(n{m) p. y.-n{i
)

legendre_pc=: 3 : 0
r=. 1   if. y.=0 do. r return. end.
s=. 0 1 if. y.=1 do. s return. end.
i=. 1
while. y. >: i=. >:i do.
  p=. r
  r=. s
  s=. (((<:+:i)*0,r) - (<:i)*p,0 0 ) % i
end.
)

=============================================================================

NB. Bessel J2
BesselJ2=: 3 : 0
t=. 1r128p1 * i.129
r=. (cos +:t) * cos (sin t) */ y.
(+/ r * 1,(127$4 2),1) % 384
)

j=. ''
j=. j,' JGRAPHICS LINE LINES LINESTYLE LINEPATTERN AREA DOT POINT POLAR STICK'
j=. j,' BAR BARS ERRORBAR HILO PIE RADAR DENSITY'
j=. j,' LINE3D STICK3D WIRE SURFACE SURFACEREV SURFACES'
j=. j,' LINE1 LINE2 LINE3 LINE4 CURVEFIT LEGENDRE PRIMERACE PAT1 PAT2 POLY'
j=. j,' WIRES GAMMA DRUM'
j=. j,' SURFACE1 SOLID SPHERE STELLATE'
PLOTNAMES=: ;:j

PLOTALL=: 'D', each PLOTNAMES

DAREA=: 0 : 0
year=.  1914 1928 1938 1948 1958 1964 1967
manu=.  125000 90700 89500 134400 94900 84900 83900
farm=.  57000 62439 39022 48252 40164 41563 37798
cler=.  3249 4558 4423 6120 6219 7395 8013
prof=.  4366 7729 11270 31636 32237 32035 33574
dat=.   100*|. (+/\  %"1  +/) prof, cler, farm,:manu
pd 'type area'
pd 'backcolor lightgray'
pd 'forecolor black'
pd 'gridcolor gray'
pd 'itemcolor blue,red,green,magenta'
pd 'xlabel ',":year
pd 'title Employment Trends by Percentage'
pd 'grids 1 0'
pd dat
pd 'show'
)

NB. LegendText 'Professional';'Clerical';'Farming';'Manufacturing'

DBAR=: 0 : 0
pd 'new'
pd 'type bar'
pd 'border 0'
pd 'xlabel "1991" "1992" "1993" "1994" "1995" "1996"'
pd 'title Bar Chart'
pd 0.6 0.8 1* _0.5+?3#,:3 3 4 7 7 4
pd 'show'
)

DBARS=: 0 : 0
dat=. 0.6 0.8 1* >:?3#,:3 3 4 7 7
xlab=. '"1991" "1992" "1993" "1994" "1995"'

pd 'new'
pd 'backcolor darkgreen'

pd 'new 20 520 460 460'
pd 'type bar'
pd 'ytic 1 0'
pd 'xlabel ',xlab
pd 'title Simple Bars'
pd {.dat

pd 'new 520 520 460 460'
pd 'type bar'
pd 'ytic 1 0'
pd 'xlabel ',xlab
pd 'title Grouped Bars'
pd dat

pd 'new 20 20 460 460'
pd 'type sbar'
pd 'ytic 2 0'
pd 'xlabel ',xlab
pd 'title Stacked Bars'
pd dat

pd 'new 520 20 460 460'
pd 'type fbar'
pd 'ytic 2 0'
pd 'xlabel ',xlab
pd 'title Floating Bars'
pd dat

pd 'show'
)

DCONTOUR=: 0 : 0
pd 'new'
pd 'backcolor white'
pd 'type contour'
dat=. 3 4 6 5 5,2 3 6 4 5, 2 2 4 2 3, 0 0 1 2 3,: 1 1 0 1 3
pd dat
NB. pd (sin */ cos) steps 0 6 10
pd 'show'
)

DCURVEFIT=: 0 : 0
NB. try changing y
y=. 3 2 5 7 6 5 7 4
x=. i.#y
dat=. x,:y
m=. cubicspline dat
x2=. steps 0,(<:#y),50
y2=. m interspline x2
y3=. (3 lsfit dat) p. x2
y4=. (5 lsfit dat) p. x2
pd 'title Spline vs Cubic vs Quintic'
pd 'backcolor lightgray'
pd 'type point'
pd x;y
pd 'type line'
pd x2;y2,y3,:y4
pd 'show'
)

DDENSITY=: 0 : 0
pd 'new'
pd 'backcolor teal'
dat=. (;];*/&sin~) steps _2 2 30
pd 'new 0 80 500 840'
pd 'type density'
pd 'bandcolor grayscale'
pd 'titlefont arial 50 italic'
pd 'title Gray-Scale Density'
pd dat
pd 'new 500 80 500 840'
pd 'type density'
pd 'titlefont arial 50 italic'
pd 'title Color-Coded Density'
pd dat
pd 'show'
)

DDOT=: 0 : 0
pd 'new'
pd 'type dot'
pd 'backcolor white'
pd 'title Dots'
pd ({.;{:)|: +.(%: * r.) p: i.4000
pd 'show'
)

DDRUM=: 0 : 0
NB. Vibrating Drum
NB. try modifying z coordinate of viewsize
NB. e.g. viewsize 1 1 0.2
NB.      viewsize 1 1 0.01
R=. steps 0 1 43
T=. steps 0 2p1 60
X=. R */ cos T
Y=. R */ sin T
Z=. (BesselJ2 R*11.6) */ cos +:T
pd 'new'
pd 'backcolor black;textcolor teal'
pd 'textfont arial 50 italic'
pd 'textc 800 950 Vibrating Drum'
pd 'plotbox 0 40 1000 840'
pd 'noaxes;type surface'
pd 'edgecolor 2{GRAYSCALE'
pd 'backcolor black;color rbclr'
pd 'viewpoint 3 3 1'
pd 'viewsize 1 1 0.05'
pd X;Y;Z
pd 'show'
)

DERRORBAR=: 0 : 0
mid=. _5 + (?21#10) + 10*sin steps 0 3 20
hi=. mid+0.5+?21#5
lo=. mid-0.5+?21#5
pd 'new'
pd 'title Error Bar'
pd 'titlefont arial 50 italic'
pd 'backcolor lightgray'
pd 'color blue'
pd 'type errorbar'
pd  hi,mid,:lo
pd 'show'
)

DGAMMA=: 0 : 0
gamma=. ! @ <:
real=. {. @ +.
x=. steps _3.5 4.5 40
y=. steps _1 1 40
z=. real gamma x j./ y
dat=. _3 >. 12 <. z
txt=. 'Gamma function near origin of complex plane'
pd 'new'
pd 'type surface'
pd 'backcolor lightgray'
pd 'textfont arial 40 italic'
pd 'textc 650 75 ',txt
pd 'color bgclr'
pd 'noaxes'
pd 'viewpoint _1 _1.5 0.5'
pd dat
pd'show'
)

DGRID=: 0 : 0
dat=. (] ; sin@cos,:sin@*:) 0.05*i.80
pd 'new'
pd 'backcolor white'
pd 'type line'
pd 'border 0'
pd 'frame 1'
pd 'grids 1'
pd 'title Just Testing...'
pd dat
pd 'show'
)

DHILO=: 0 : 0
dat=. (-200 <.#DATHILO) {. DATHILO
dat=. ',' cut"1 dat
dat=. 2 3{"1 dat
dat=. |: ". &> dat
pd 'new'
pd 'type hilo'
pd 'itemcolor darkblue'
pd 'title March 1996 Deutsche Mark Future'
pd 'titlefont arial 50'
pd dat
pd 'show'
)

DJGRAPHICS=: 0 : 0
dat=. 0.1*_80+i.160
dat=. (] ; sin , (%&7 ^ 3:) ,: [: tan [: ^&3 cos) dat
pd 'new'
pd 'backcolor teal'
pd 'textfont arial 100 bold italic'
pd 'textcolor lightblue'
pd 'textc 500 950 J Plot'
pd 'new 150 50 700 700'
pd 'type line'
pd 'backcolor lightgray'
pd 'gridcolor gray'
pd 'frame 1'
pd 'grids 1'
pd dat
pd 'show'
)

DLEGENDRE=: 0 : 0
x=. steps _1 1 100
y=. (legendre_pc"0 [ 2 4 6 8) p."1 x
pd 'new'
pd 'type line'
pd 'backcolor lightgray;'
pd 'color blue, red, darkgreen, magenta'
pd 'title Legendre Polynomials of order 2 4 6 8'
pd 'titlefont arial 40 italic'
pd x;y
pd 'show'
)

DLINE3D=: 0 : 0
u=. 0.5*steps 0 50p1 500
r=. sin steps (o. (,-.)0.1), 500
x=. r*sin u
y=. r*cos u
z=. 0.1*u
pd 'new'
pd 'backcolor lightgray'
pd 'color red'
pd 'viewpoint 1.3 0.9 0.25'
pd 'viewsize 1'
pd x;y;z
pd 'show'
)

DLINE=: 0 : 0
dat=. sin *: 0.05*i.60
pd 'new'
pd 'type line'
pd 'backcolor lightgray'
pd 'titlefont arial 50 italic'
pd 'title Simple Line'
pd 'captionfont arial 40 italic'
pd 'xcaption x-axis'
pd 'ycaption y-axis'
pd dat
pd 'show'
)

DLINES=: 0 : 0
opt=. 'type line;titlefont arial 40 italic;backcolor lightgray'

pd 'new'
pd 'backcolor teal'

pd 'new 20 520 460 460'
pd opt
pd 'ytic 40 1'
pd 'xtic 20 2'
pd 'xcaption x1'
pd 'ycaption one'
pd 'title Simple Line'
pd  *: 0.1*i.160

pd 'new 520 520 460 460'
pd opt
pd 'xcaption x2'
pd 'ycaption two'
pd 'title x cos(x) vs x sin(x)'
pd ((* cos) ; (* sin)) range 0 8 0.1

fn=. (cos@+: * cos,sin) "0
dat=. ({.;{:) |: fn range 0 2p1 0.05
pd 'new 20 20 460 460'
pd opt
pd 'ytic 0.5 4'
pd 'xtic 0.5 4'
pd 'xcaption x3'
pd 'ycaption three'
pd 'title r=cos 2t for t in [0, 2pi]'
pd dat

pd 'new 520 20 460 460'
pd opt
pd 'ytic 0.5 4'
pd 'xtic 0.5 4'
pd 'xcaption x4'
pd 'ycaption four'
pd 'title sin vs cos, sin*cos'
pd (sin;cos,:sin*cos) range 1 10 0.1
pd 'show'
)

DLINESTYLE=: 0 : 0
dat=. ((* cos) ,: (* sin)) range 0 8 0.1
dat=. ((* cos) ; (* sin@(1.1&*)) ,: (* sin)) range 0 8 0.1
opt=. 'backcolor lightgray;color red,darkgreen;textcolor darkblue;type line'

pd 'new'
pd 'backcolor teal'

pd 'new 20 520 460 460'
pd opt
pd 'title Default Options'
pd 'titlefont arial 40 italic'
pd dat

pd 'new 520 520 460 460'
pd opt
pd 'title Move X-axis'
pd 'xint -4;'
pd 'titlefont arial 40 italic'
pd dat

pd 'new 20 20 460 460'
pd opt
pd 'title No Tics, Labels'
pd 'tics 0;labels 0'
pd 'titlefont arial 40 italic'
pd dat

pd 'new 520 20 460 460'
pd opt
pd 'frame 1'
pd 'xtic 2 4;ytic 4 1'
pd 'grids 1'
pd 'gridcolor gray'
pd 'title Framed with Grid'
pd 'titlefont arial 40 italic'
pd dat

pd 'show'
)

DLINEPATTERN=: 0 : 0
dat=. (;(sin+cos),sin,cos,(sin@+:),:cos&+:) steps 0 4 50
pd'new'
pd 'penstyle 0 1 2 3 4'
pd 'backcolor lightgray'
pd 'color green,red,blue,magenta,brown'
pd 'title Line Patterns'
pd dat
pd 'show'
)

DLINE1=: 0 : 0
dat=. (;(sin,:cos)@:^) range _1 1.95 0.05
pd 'frame 1'
pd 'grids 1'
pd 'backcolor lightgray'
pd 'gridcolor gray'
pd 'title sin(exp) vs cos(exp)'
pd dat
pd 'show'
)

DLINE2=: 0 : 0
wiggle=. 4       NB. try 5 and higher
points=. 300     NB. increase for smoothness
x=. (3 % <:points) * i.points
fn=. +/@((0.9&^) * cos@((3&^ * (+/&x))))@i.
pd 'frame 1'
pd 'backcolor lightgray;color blue'
pd x;fn wiggle
pd 'show'
)

DLINE3=: 0 : 0
dat=. (;*cos) range _400 800,(0.99*1p1)
pd 'new'
pd 'type line'
pd 'frame 1'
pd 'backcolor lightgray;color blue'
pd 'title x * cos x'
pd dat
pd 'show'
)

DLINE4=: 0 : 0
dat=. ((*sin);(*cos)) range 0 600,0.99*1r2p1
pd 'new'
pd 'type line'
pd 'frame 1'
pd 'backcolor lightgray;color blue'
pd dat
pd 'show'
)

DPAT1=: 0 : 0
NB. try changing pqr
NB. e.g. 16 16 8
NB. e.g. 6 16 16
'p q r'=. 16 8 8
T=. steps 0 2p1 200
U=. T + (sin q*T)%p
V=. 2 + (sin r*T)%2
dat=. (V*cos U);V*sin U
pd 'noaxes'
pd 'backcolor darkblue;color red'
pd"1 (}.int01 30) *each / dat
pd 'show'
)

DPAT2=: 0 : 0
NB. try varying p1, p2, s:
NB. e.g. s=206, s=400, s=404
p1=. 101
p2=. 102 103
s=. 305
dat=. steps 0 2p1, s
pd 'new'
pd 'noaxes'
pd 'backcolor darkblue'
pd 'itemcolor magenta,red'
pd (cos p1*dat);sin p2*/dat
pd 'show'
)

DPIE=: 0 : 0
pd 'new'
pd 'backcolor lightgray'
pd 'textfont Arial 70 italic'
pd 'textcolor blue'
pd 'text 120 900 Elbow Room'

pd 'new 0 0 500 800'
pd 'textfont Arial 50 italic'
pd 'text 310 860 Population'
pd 'clear 0'
pd 'color red,blue,green,yellow,teal'
pd 'type pie'
pd  25 1008 684 117 229

pd 'new 470 200 500 800'
pd 'textfont Arial 50 italic'
pd 'text 330 860 Land Area'
pd 'clear 0'
pd 'color red,blue,green,yellow,teal'
pd 'type pie'
pd 'key Canada China India Japan USA'
pd 'keystyle 1 2'
pd  9972 9587 3116 381 9160
pd 'show'
)

DPOINT=: 0 : 0
dat=. (;(sin*cos),sin,:cos) steps _1 3 30
pd 'new'
pd 'title Point'
pd 'type point'
pd dat
pd 'show'
)

DPOLAR=: 0 : 0
x1=. steps 0 20 1000
opt=. 'polar 1;backcolor lightgray;color red,darkgreen'
opt=. opt,';textcolor darkblue;type line;titlefont arial 40 italic'

pd 'new'
pd 'backcolor teal'

pd 'new 20 520 460 460'
pd opt
pd 'title sin vs cos'
pd (sin;cos) x1

pd 'new 520 520 460 460'
pd opt
pd 'title sin @ (%&5)'
pd (sin @ (%&5) ; ]) x1

pd 'new 20 20 460 460'
pd opt
pd 'title r squared'
pd (*: ; ]) x1

pd 'new 520 20 460 460'
pd opt
pd 'title sin @ +:'
pd (sin@+: ; ]) x1

pd 'show'
)

DPOLY=: 0 : 0
reg=. +.@(_1:^2:*i.%])
triangulate=. [ (,"2) 2: ]\ ] , {.@]
det=. -/ . *
area=. det@border
border=. (,.%@!@{:@$)"2
treg=. 1r2 1r3 triangulate reg 4
dat=. ({."1;{:"1) treg
pd 'new'
pd 'type poly'
pd 'frame 1'
pd 'backcolor lightgray'
pd 'itemcolor green, blue, red, yellow'
pd dat
pd 'show'
)

DPRIMERACE=: 0 : 0
NB. primes = 3 (mod 4) minus
NB. primes = 1 (mod 4)
NB. try: (begin,end) =       0, 100000
NB.             or      808000, 812000
primerace=: [: +/\ _2: + 4: | p:
begin=. 50000
end=. 52000
dat=. begin }.each (;primerace) i.end
pd 'new'
pd 'title Prime Race'
pd 'backcolor lightgray'
pd 'gridcolor black'
pd 'frame 1'
pd dat
pd'show'
)

DRADAR=: 0 : 0
j=. ,: 0.4 0.5 0.5 0.6 0.4
j=. j, 0.6 0.6 0.6 0.5 0.9
j=. j, 1 0.8 0.6 0.3 0.8
dat=. |:j
pd 'new'
pd 'type radar'
pd 'title Radar Plot'
pd 'titlefont arial 50 italic'
pd 'backcolor lightgray'
pd 'gridcolor gray'
pd 'forecolor gray'
pd 'itemcolor red,blue,green'
pd 'grids 1'
pd dat
pd 'show'
)

DSOLID=: 0 : 0
cut=. <"2 @ (0 1&|:)
CLR=: RED,GREEN,BLUE,FUCHSIA,AQUA,YELLOW,TEAL,BROWN
pd 'new'
pd 'backcolor lightgray'
pd 'use 50 600 250 350;clear 0'
pd 'type poly;color clr'
pd 'noaxes;viewsize 1 1 1'
pd  cut cube''
pd 'use 700 600 250 350'
pd  cut tetrahedron''
pd 'use 50 50 250 350'
pd  cut octahedron''
pd 'use 700 50 250 350'
pd  cut dodecahedron''
pd 'use 288 180 425 640'
pd  cut icosahedron''
pd 'show'
)

DSPHERE=: 0 : 0
P=. steps 0 1p1 30
T=. steps 0 2p1 40
X=. (sin P) */ cos T
Y=. (sin P) */ sin T
Z=. (cos P) +/ T*0
pd 'new'
pd 'backcolor blue'
pd 'new 150 0 700 1000'
pd 'clear 0'
pd 'type surface'
pd 'noaxes;viewpoint 2 2 1.5;color grayscale'
pd X;Y;Z
pd 'show'
)

DSTELLATE=: 0 : 0
pd 'new'
pd 'backcolor blue'
pd 'type poly;noaxes'
pd 'use 25 50 350 500;clear 0'
pd 'viewsize 1 1 1'
pd 'viewpoint 1 2 1'
pd 'color rbclr'
pd cutxyz ,/0.3 stellate dodecahedron ''
pd 'use 325 50 700 950'
pd cutxyz ,/0.6 stellate icosahedron ''
pd 'show'
)

DSTICK=: 0 : 0
pd 'new'
pd 'type stick'
pd 'backcolor lightgray'
pd 'itemcolor blue'
pd 'title Stick'
pd (;(*cos)) steps _4 7.8 150
pd 'show'
)

DSTICK3D=: 0 : 0
u=. steps 0 6.5 150
r=. sin steps (o. (,-.)0.1), <:#u
x=. r*cos u
y=. r*(sin*cos) u
z=. 0.1*u
pd 'new'
pd 'type stick'
pd 'backcolor lightgray'
pd 'color darkblue'
pd 'viewpoint _1 2 1'
pd 'viewsize 1 1 0.75'
pd x;y;z
pd 'show'
)

DSURFACE=: 0 : 0
dat=. (];];sin@*/)~steps _2 2 30
pd 'type surface'
pd 'backcolor lightgray'
pd dat
pd 'show'
)

DSURFACE1=: 0 : 0
NB. example of polar vs cartesian coordinates
NB. default polar coordinates (much better for this plot)
NB. uncomment second dat for cartesian coordinates
T=. steps 0 2p1 60
R=. steps 0.1 2 15
dat=. R ((*/cos);(*/sin);sin@+:@]) T
X=. steps _2 2 40
NB. dat=. (*%+&*:)"0/~X
pd 'new'
pd 'backcolor black'
pd 'new 100 0 800 1000'
pd 'type surface'
pd 'color rbclr;backcolor black'
pd 'noaxes;viewpoint 2 4 1.5'
pd dat
pd 'show'
)

DSURFACEREV=: 0 : 0
X=. steps _2 1.9 20
dat=. cutxyz 25 surfacerev (2+*:X);X
CLR=: (RED,:YELLOW) hue _8}.8}.int01 32
pd 'new'
pd 'backcolor teal;color CLR'
pd 'type surface'
pd dat
pd 'show'
)

DSURFACES=: 0 : 0
pd 'new'
pd 'backcolor teal'

pd 'new 20 520 460 460'
pd 'boxed 0'
pd 'viewpoint 0 _2 1.75'
pd 'viewsize 1'
pd 'type surface'
pd (];];^@-@+/&:*:)~steps _2 2 40

C=. 0,.(255<.8*i.33),.64
T=. steps 0 2p1 50
X=. (sin T) */ 5+sin T
Y=. (cos T) */ 5+sin T
Z=. cos T
pd 'new 20 20 460 460'
pd 'backcolor darkblue'
pd 'new 70 70 360 360'
pd 'clear 0'
pd 'boxed 0'
pd 'mesh 0'
pd 'bandcolor ',":,C
pd 'type surface'
pd 'viewpoint 3 3 1.5'
pd 'viewsize 1 1 0.1'
pd  X;Y;Z

T=. steps 0 2p1 30
U=. steps _1p1 1p1 30
X=. (cos T)*/>:0.2*sin U
Y=. (sin T)*/>:0.2*sin U
Z=. 0.2*(sin T)*/cos U
pd 'new 520 520 460 460'
pd 'type surface'
pd 'backcolor black'
pd 'boxed 0'
pd 'bandcolor bgclr'
pd 'viewpoint 3 3 2.5'
pd X;Y;Z

pd 'new 520 20 460 460'
pd 'type surface'
pd 'viewpoint 2 _1.2 1.5'
pd (];sin@];cos@*/)~ steps 0 3 30
pd 'show'
)

DWIRE=: 0 : 0
x=. | steps _8 8 40
d=. % + 0: = ]
x=. (sin d ]) (+/&.*:)/~x
pd 'boxed 0'
pd 'type wire'
pd 'backcolor lightgray'
pd 'bandcolor darkblue'
pd 'title (sin x) % x'
pd 'viewpoint 2.5 3 0.75'
pd x
pd 'show'
)

DWIRES=: 0 : 0
dat=. (] % ^@+&*:)"0/~ steps _2.4 2.4 51
pd 'boxed 0'
pd 'type wire'
pd 'backcolor black'
pd 'bandcolor |.bgclr'
pd 'viewpoint 2 _5 0.7'
pd 'viewsize 1 1 0.5'
pd dat
pd 'show'
)
