NB. built from project: ~Classes\graphics\bmp\bmp
NB. supports bitmap files
NB.             i.e. *.bmp files  (in DIB format)
NB.
NB. readbmp and writebmp use RGB values (single numbers)
NB. writebmp also accepts RGB triples
NB.
NB. main functions:
NB.
NB.   viewrgb      view numeric matrix of RGB data
NB.   viewbmp      view bitmap file
NB.
NB.   readbmp      read bitmap file, returning RGB data
NB.   writebmp     write bitmap file from RGB data
NB.
NB.   readbmphdr   read header from bitmap file

cocurrent 'z'


NB. =========================================================
NB.*readbmp v read bitmap file, returning RGB data
readbmp=: 3 : 0

try. dat=. 1!:1 boxopen y.
catch. 'file read error' return. end.

if. -. 'BM'-:2{.dat do. 'not a bitmap file' return. end.

toi=. 256&#.@(a.&i.)@(|."1)
bits=. toi 28 29 {dat

if. toi 30 31 32 33{dat do.
  'compressed format not supported' return.
end.

'off shdr cls rws'=. toi (10+i.4 4){dat

pal=. off {. dat
dat=. off }. dat

if. bits e. 1 4 8 do.
  pal=. toi _4 }: \ (shdr+14) }. pal
  dat=. , ((#~ ^.&256) 2^bits) #: a. i. dat
  pal {~ |. (rws,cls){.(rws,cls+(32%bits)|-cls) $ dat
elseif. bits=24 do.
  cl4=. 4 * >. (3*cls) % 4
  |. (rws,cls) {. 256 #. |."1 a.i. _3 [\"1 (rws,cl4) $ dat
elseif. 1 do.
  'only 1,4,8 and 24-bit bitmaps supported, this is ',(":bits),'-bit'
end.
)

NB. =========================================================
NB.*readbmphdr v read header from bitmap file
NB. returns:  bitsize, rows, columns
readbmphdr=: 3 : 0

try. dat=. 1!:1 boxopen y.
catch. 'file read error' return. end.

if. -. 'BM'-:2{.dat do. 'not a bitmap file' return. end.

toi=. 256&#.@(a.&i.)@(|."1)
bits=. toi 28 29 {dat

if. toi 30 31 32 33{dat do.
  'compressed format not supported' return.
end.

'off shdr cls rws'=. toi (10+i.4 4){dat
bits,rws,cls
)

NB. =========================================================
NB.*writebmp v write bmp file from RGB data
NB.
NB. Form:  data writebmp filename [;minimum bitsize]
NB.
NB. picks appropriate bit size of 4 8 or 24, subject
NB. to optional minimum bit size.

writebmp=: 4 : 0

dat=. x.
'file min'=. 2 {. (boxopen y.), <0

if3=. (3=#$dat) *. 3={:$dat

sbmp=. (-if3) }. $dat
pal=. /:~ ~. ,/ dat
bit=. 4 8 24 {~ +/ 16 256 < # pal
bit=. min >. bit

if. bit e. 4 8 do.
  dat=. pal i. dat
  if. bit=4 do.
    dat=. 16 #. _2[\"1 dat
  else.
    pal=. 256 {. pal
  end.
  spal=. #pal
  if. if3 do.
    pal=. |."1 pal
  else.
    pal=. 256 256 256 #: pal
  end.
  pal=. , a. {~ pal ,. 0
  xsbmp=. ($dat) + 0 1 * 4|-$dat
  bmp=. xsbmp{.dat
else.
  pal=. ''
  spal=. 0
  if. if3 do.
    dat=. |."1 dat
  else.
    dat=. 256 256 256 #: dat
  end.
  bmp=. ,"2 dat
  bmp=. ((1 4)* >. ($bmp) % (1 4)) {. bmp
end.

bmp=. ,|. bmp{ a.
sdat=. $bmp

j=. (|.sbmp),(256 #. 0, bit, 0 1),0,sdat,0 0,2#spal
b=. (54+(4*spal)+sdat),0,(54+4*spal),40,j
head=. 'BM',,a.{~,|."1 (4#256)#:b

(head,pal,bmp) 1!:2 boxopen file
)

NB. =========================================================
NB.*viewbmp v bitmap viewer using isipicture control
NB.
NB. x. = [parentid [;parentname]]
NB. y. = filename(s)
NB.
NB. e.g.  viewbmp jpath '~system\examples\data\j.bmp'
viewbmp=: 3 : 0
'' viewbmp y.
:
if. -. IFWIN32 do.
  wdinfo 'viewbmp';'Not available' return.
end.

if. 0=#x. do. n=. }. p=. 'jbitmap'
else. 'p n'=. 2 $ boxopen x. end.

f=. y.
p=. p -. ' '

if. (<p) e. <;._2 wd 'qp;' do.
  wd 'psel ',p
else.
  dim=. |. }. readbmphdr f
  dim=. <. 150 * ( % >. / ) dim
  wd 'pc ',p,' closeok;pn *',n
  wd 'xywh 0 0 ',":dim
  wd 'cc p0 isipicture rightmove bottommove'
  wd 'pas 0 0;pcenter;pmove 150 5 -1 -1'
end.
wd 'set p0 "',f,'";ptop;pshow;'
)

NB. =========================================================
NB.*viewrgb v view numeric matrix of RGB data
NB.                  (writes bitmap file, then views it)
NB. x. = [parent]
NB. y. = matrix
viewrgb=: 3 : 0
'' viewrgb y.
:
f=. 'temp\t1.bmp'
y. writebmp f
x. viewbmp f
)

NB. =========================================================
NB.*hue v generate color from color set
NB. x. is color set
NB. y. is values from 0 to 1, selecting color
hue=: 4 : 0
y=. y.*<:#x.
b=. x. {~ <.y
t=. x. {~ >.y
k=. y-<.y
(t*k)+b*-.k
)

